# This file is part of the "FDN tools" project.
# You can use and redistribute it under the terms of the
# GNU General Public License.
# (c) French Data Network 1997-2005
# Authors and contributors:
# - Benjamin Bayart
package FDN::Etebac::Etebac3;

use strict;
use Exporter;
use FDN::Common::Logger;

our @ISA = ( 'Exporter' ) ;
our @EXPORT = qw (
		check_operations
		
		open_prelevement
		print_prelevement
		close_prelevement
		
		open_virement
		print_virement
		close_virement

		analyse_rejet_prelevement
                );
our @EXPORT_OK = qw ();

our $debug = 0;
sub __debug($) {
	return if ! $debug;
	debug $_[0];
}

sub check_operations();

BEGIN {
#	check_operations();
}

my $ops = {};
my $fichier;

$ops->{'prelevement'} = {
	'direction' => 'A',
	'longueur' => 160,
	'bordereau' => {
		'ordinaire' => '08',
		'accelere'  => '85',
		'default'   => 'ordinaire'
	},
	'open' => {
		'code' => '03',
		'obligatoires' => [
			'emetteur/6N',
			'date/5N',
			'raison-soc/24AN',
			'dst/BQ'
		],
		'defauts' => {
			'monnaie' => 'E',
			'domiciliation' => '',
			'sequence' => undef,
		},
		'champs' => [
			'code',
			'operation',
			'PAD8',
			'emetteur',
			'PAD7',
			'date',
			'raison-soc', #Quid?
			'sequence', #Quid?
			'PAD17',
			'PAD2',
			'monnaie',
			'PAD5',
			'guichet/dst',
			'compte/dst',
			'PAD16',
			'PAD31',
			'banque/dst',
			'PAD6'
		],
	},
	'print' => {
		'code' => '06',
		'obligatoires' => [
			'nom/24AN',
			'src/BQ',
		],
		'defauts' => {
			'monnaie' => ' ',
			'domiciliationbq/src' => '',
			'libelle' => ''
		},
		'champs' => [
			'code',
			'operation',
			'PAD8',
			'emetteur',
			'reference',
			'nom',
			'domiciliationbq/src',
			'PAD2',
			'monnaie',
			'PAD5',
			'guichet/src',
			'compte/src',
			'montant',
			'libelle',
			'banque/src',
			'PAD6'
		],
	},
	'close' => {
		'code' => '08',
		'obligatoires' => [
		],
		'champs' => [
			'code',
			'operation',
			'PAD8',
			'emetteur',
			'PAD12',
			'PAD24',
			'PAD24',
			'PAD8',
			'PAD5',
			'PAD11',
			'total',
			'PAD31',
			'PAD5',
			'PAD6'
		],
	},
	'champs' => {
		'code' => '2N', # Tete/detail/pied
		'operation' => '2N', # Sais pas. Bordereau?
		'emetteur' => '6AN', # Sais pas, numero national?
		'date' => '5N', # JJMMA
		'raison-soc' => '24AN', # Sais pas
		'sequence' => '7N', # Sais pas
		'domiciliation' => '2AN', # Sais pas
		'guichet' => '5N',
		'compte' => '11BQ',
		'banque' => '5N',
		'reference' => '12AN',
		'nom' => '24AN',
		'domiciliationbq' => '24AN',
		'monnaie' => '1AN',
		'montant' => '16N',
		'libelle' => '31AN',
		'total' => '16N',
	},
};

sub open_prelevement($) {
	my ( $args ) = @_;
	my $vals = check_args('prelevement','open',$args);
	my $res = {};
	$res->{'vals'} = $vals;
	$res->{'nbligs'} = 0;
	$res->{'total'} = 0;
	my $fichier;
	faire_ligne('prelevement','open',$res->{'vals'},$vals);
	return $res;
}

sub print_prelevement($$) {
	my ( $prl, $args ) = @_;
	my $vals = check_args('prelevement','print',$args);
	faire_ligne('prelevement','print',$prl->{'vals'},$vals);
	$prl->{'nbligs'}++;
	$prl->{'total'} += $vals->{'montant'};
}

sub close_prelevement($$) {
	my ( $prl, $args ) = @_;
	my $vals = check_args('prelevement','close',$args);
	$vals->{'total'} = $prl->{'total'};
	faire_ligne('prelevement','close',$prl->{'vals'},$vals);
	return $fichier;
}

$ops->{'virement'} = {
	'direction' => 'A',
	'longueur' => 160,
	'bordereau' => {
		'ordinaire' => '02',
		'verifier' => '29',
		'apl' => '22', # Aide personnalisee au logement
		'e-2' => '28',
		'e-3' => '27',
		'tresorerie' => '76', # Virement de tresorerie (VSOT)
		'default' => 'ordinaire'
	},
	'open' => {
		'code' => '03',
		'obligatoires' => [
			'emetteur/6N',
			'date/5N',
			'raison-soc/24N',
			'src/BQ',
		],
		'defauts' => {
			'monnaie' => 'E',
			'ifsiret' => '',
			'siret' => '',
		},
		'champs' => [
			'code',
			'operation',
			'PAD8',
			'emetteur',
			'code-ccd', # Utilise par les virement E-3
			'PAD6',
			'date',
			'raison-soc',
			'sequence',
			'PAD17',
			'PAD2',
			'monnaie',
			'PAD5',
			'guichet/src',
			'compte/src',
			'identification',
			'PAD31',
			'banque/src',
			'PAD6'
		],
	},
	'print' => {
		'code' => '06',
		'obligatoires' => [
			'nom/24AN',
			'dst/BQ',
		],
		'defaults' => {
			'domiciliationbq/src' => '',
			'libelle' => '',
		},
		'champs' => [
			'code',
			'operation',
			'PAD8',
			'emetteur',
			'reference',
			'nom',
			'domiciliationbq/dst',
			'non-resident', # Quid?
			'PAD8',
			'guichet/dst',
			'compte/dst',
			'montant',
			'libelle',
			'banque/dst',
			'PAD6',
		],
	},
	'close' => {
		'code' => '08',
		'obligatoires' => [
		],
		'defauts' => {
		},
		'champs' => [
			'code',
			'operation',
			'PAD8',
			'emetteur',
			'PAD84',
			'total',
			'PAD42',
		],
	},
	'champs' => {
		'code' => '2N', # Tete/detail/pied
		'operation' => '2N', # Sais pas. Bordereau?
		'emetteur' => '6AN', # Sais pas, numero national?
		'date' => '5N', # JJMMA
		'raison-soc' => '24AN', # Sais pas
		'sequence' => '7N', # Sais pas
		'domiciliation' => '2AN', # Sais pas
		'guichet' => '5N',
		'compte' => '11BQ',
		'banque' => '5N',
		'reference' => '12AN',
		'nom' => '24AN',
		'domiciliationbq' => '20AN',
		'monnaie' => '1AN',
		'montant' => '16N',
		'libelle' => '31AN',
		'total' => '16N',
		'identification' => '16AN', # Identification du donneur d'ordre, optionnel
		'non-resident' => '4AN',
		'code-ccd' => '1AN',
	},
};

sub open_virement($) {
	my ( $args ) = @_;
	my $vals = check_args('virement','open',$args);
	my $res = {};
	$res->{'vals'} = $vals;
	$res->{'nbligs'} = 0;
	$res->{'total'} = 0;
	$fichier = '';
	faire_ligne('virement','open',$res->{'vals'},$vals);
	return $res;
}

sub print_virement($$) {
	my ( $vir, $args ) = @_;
	my $vals = check_args('virement','print',$args);
	faire_ligne('virement','print',$vir->{'vals'},$vals);
	$vir->{'nbligs'}++;
	$vir->{'total'} += $vals->{'montant'};
}

sub close_virement($$) {
	my ( $vir, $args ) = @_;
	my $vals = check_args('virement','close',$args);
	$vals->{'total'} = $vir->{'total'};
	faire_ligne('virement','close',$vir->{'vals'},$vals);
	return $fichier;
}


$ops->{'rejet-prelevement'} = {
	'direction' => 'R',
	'longueur' => 240,
	'bordereau' => {
		81 => 'normal',
	},
	'open' => {
		'code' => 31,
		'champs' => [
			'code',
			'numero',
			'PAD2',
			'date',
			'monnaie',
			'PAD4',
			'banque/dst',
			'guichet/dst',
			'compte/dst',
			'nom/dst',
			'emetteur',
			'banque/master',
			'banque/src',
			'guichet/src',
			'compte/src',
			'nom/src',
			'emetteur',
			'PAD111'
		],
		
	},
	'read' => {
		'code' => '34',
		'champs' => [
			'code',
			'numero',
			'operation',
			'daterejet',
			'sit', # Quid?
			'PAD4',
			'banque/dst',
			'guichet/dst',
			'compte/dst',
			'nom/dst',
			'emetteur',
			'banque/master',
			'banque/src',
			'guichet/src',
			'compte/src',
			'nom/src',
			'refrejet',
			'domicilbq/src',
			'libelle',
			'PAD31',
			'dateinit',
			'reference',
			'coderejet',
			'montant'
		],
	},
	'close' => {
		'code' => '39',
		'champs' => [
			'code',
			'numero',
			'operation', # 21
			'date',
			'PAD212',
			'montant'
		],
	},
	'champs' => {
		'code' => '2N',
		'numero' => '6N', # Numero d'enregistrement
		'date' => '6N',
		'daterejet' => '6N',
		'dateinit' => '6N',
		'monnaie' => '1AN',
		'banque' => '5N',
		'guichet' => '5N',
		'compte' => '11AN',
		'domicilbq' => '24AN',
		'nom' => '24AN',
		'emetteur' => '6N',
		'sit' => '1AN', # Quid?
		'refrejet' => ' 6AN',
		'libelle' => '31AN',
		'reference' => '6AN',
		'coderejet' => '2N',
		'montant' => '12N',
	},
	'code' => {
		'12' => "RIB INEXPLOITABLE",
		'14' => "COMPTE SOLD",
		'15' => "COMPTE VIR",
		'16' => "DESTINATAIRE NON RECONNU",
		'18' => "EMETTEUR NON PRCIS",
		'20' => "PROVISION INSUFFISANTE",
		'31' => "PAS D'ORDRE DE PAYER",
		'32' => "DCISION JUDICIAIRE",
		'33' => "DPT DE BILAN",
		'34' => "OPPOSITION SUR COMPTE",
		'35' => "TITULAIRE DCD",
		'70' => "TIRAGE CONTEST",
		'71' => "REU  TORT/DJ RGL",
		'73' => "MONTANT CONTEST",
		'74' => "DATE D'CHANCE CONTESTE",
		'75' => "DEMANDE DE PROROGATION",
		'76' => "RCLAMATION TARDIVE",
		'77' => "OPRATION DJ REJETE",
		'78' => "OPRATION D'ORIGINE NON PARVENUE",
		'80' => "RGULARISATION D'AP",
		'88' => "BANQUE HORS CHANGES"
	},
	'code-detail' => {
		'12' => "Anomalie au niveau d'un lment du RIB du\n".
		        "destinataire (code banque, code guichet ou\n".
			"numro de compte ne correspondant pas  des\n".
			"coordonnes utilises par la banque domiciliaire)",
		'14' => "Compte sold",
		'15' => "Compte vir",
		'16' => "Le nom du destinataire ne correspond pas au RIB\n".
			"ou n'est pas prcis",
		'18' => "metteur non prcis ou non identifiable (le numro\n".
			"national d'metteur ne correspond pas  celui\n".
			"figurant sur l'Autorisation de Prlvement)",
		'20' => "Provision non suffisante",
		'31' => "Pas d'ordre de payer (ne s'applique que pour une\n".
			"absence d'autorisation; pour les cas o le refus\n".
			"de payer mane du client, on utilise les codes 70,\n".
			"71, 73, 74 voire 75)",
		'32' => "Dcision judiciare:".
			"- rglement judiciaire\n".
			"- liquidation de biens\n".
			"- faillite personnelle\n".
			"- suspension provisoire des poursuites",
		'33' => "Dpt de bilan",
		'34' => "Compte frapp d'opposition ou compte bloqu",
		'35' => "Titulaire du compte dcd",
		'70' => "Tirage contest par le client (rejet explicite \n".
			"la demande du client)",
		'71' => "Dj rgl par le biais d'un autre moyen de paiement\n".
			"ou opration en double (rejet explicite  la demande\n".
			"du client)",
		'73' => "Montant contest par le client",
		'74' => "Date d'chance conteste par le client",
		'75' => "Demande de prorogation (rejet explicite  la demande\n".
			"du client)",
		'76' => "Rclamation tardive de l'metteur (ce code peut tre\n".
			"utilis en particulier dans le cas d'une opration\n".
			"mise  tort ou en double)",
		'77' => "Opration dj rejete (dans le cas d'AOCT)",
		'78' => "Opration d'origine non parvenue (dans le cas d'AOCT)",
		'80' => "Demande de contrepassation d'AP (Autorisation de\n".
			"Prlvement?) prsum erron",
		'88' => "Cessation de reprsentation directe ou indirecte d'un\n".
			"tablissement domiciliaire dans les systmes d'change\n".
			"interbancaire (la banque du client n'est plus relie aux\n".
			"systmes de compensation interbancaires)"
	},
};


sub analyse_rejet_prelevement() {
}


#########
# Operations generiques
#########

sub faire_ligne($$$$) {
	my ($op,$type,$global,$local) = @_;
	for my $champ ( @{$ops->{$op}->{$type}->{'champs'}} ) {
		__debug "Traitement du champ $champ";
		my $val = get_args_val($champ,$global,$local);
		ajoute_champ($op,$champ,$val);
	}
	ajoute_crlf();
}

sub get_args_val($$;$) {
	my ( $champ, $global, $local ) = @_;
	my $res = undef;
	if ( $champ =~ /^PAD\d+$/ ) {
		return '';
	}
	if ( ! defined $local ) {
		$local = {};
	}
	if ( defined $local->{$champ} ) {
		$res = $local->{$champ};
		__debug "Champ $champ pris en local: $res";
	} elsif ( defined $global->{$champ} ) {
		$res = $global->{$champ};
		__debug "Champ $champ pris en global: $res";
	} else {
		__debug "Champ $champ introuvable";
	}
	return $res;
}

sub check_args($$$) {
	my ( $op, $typerq, $args ) = @_;
	my $vals = {};
	for my $x ( @{$ops->{$op}->{$typerq}->{'obligatoires'}} ) {
		$x =~ /^(.*)\/(\d*)([A-Z]*)$/;
		my ( $nom, $len, $type ) = ( $1, $2, $3 );
		if ( defined $args->{$nom} ) {
			if ( $type eq 'AN' ) {
				$vals->{$nom} = uc $args->{$nom};
			} elsif ( $type eq 'BQ' ) {
				if ( ! defined $args->{$nom}->{'guichet'} ||
				     ! defined $args->{$nom}->{'banque'} ||
				     ! defined $args->{$nom}->{'compte'} ) {
					die "Reference compte bancaire $nom pour $typerq sur $op invalide";
				}
				$vals->{'banque/'.$nom} = $args->{$nom}->{'banque'};
				$vals->{'guichet/'.$nom} = $args->{$nom}->{'guichet'};
				$vals->{'compte/'.$nom} = uc $args->{$nom}->{'compte'};
				if ( defined $args->{$nom}->{'domiciliation'} ) {
					__debug "DOMICILIATION ".
						$args->{$nom}->{'domiciliation'}.
						" POUR ".
						'domiciliationbq/'.$nom."\n";
					$vals->{'domiciliationbq/'.$nom} =
						uc $args->{$nom}->{'domiciliation'};
				} else {
					__debug "PAS DE DOMICILIATION\n";
				}
			} else {
				$vals->{$nom} = $args->{$nom};
			}
			delete $args->{$nom};
		} else {
			die "Manque le parametre $nom pour $typerq sur $op";
		}
	}
	if ( defined $ops->{$op}->{'bordereau'} ) {
		if ( ! defined $args->{'bordereau'} ) {
			$args->{'bordereau'} = $ops->{$op}->{'bordereau'}->{'default'};
		}
		$vals->{'operation'} = $ops->{$op}->{'bordereau'}->{$args->{'bordereau'}};
		delete $args->{'bordereau'};
	}
	$vals->{'code'} = $ops->{$op}->{$typerq}->{'code'};
	for my $nom ( keys %$args ) {
		$vals->{$nom} = uc $args->{$nom};
	}
	for my $x ( keys %{$ops->{$op}->{$typerq}->{'defauts'}} ) {
		if ( ! defined $vals->{$x} ) {
			$vals->{$x} = $ops->{$op}->{$typerq}->{'defauts'}->{$x};
		}
	}
	return $vals;
}

sub check_operations() {
	for my $op ( keys %$ops ) {
		check_line($op,'open');
		if ( $ops->{$op}->{'direction'} eq 'A' ) {
			check_line($op,'print');
		} else {
			check_line($op,'read');
		}
		check_line($op,'close');
	}
}

sub check_line($$) {
	my ( $op, $type ) = @_;
	my $len = 0;
	for my $tmp ( @{$ops->{$op}->{$type}->{'champs'}} ) {
		if ( $tmp =~ /^PAD(\d+)$/ ) {
			$len += $1;
		} else {
			my $champ = $tmp;
			$champ =~ s/\/.*$//;
			my $fld = $ops->{$op}->{'champs'}->{$champ};
			if ( defined $fld ) {
				if ( $fld =~ /^(\d+)(N|AN|BQ)$/ ) {
					$len += $1;
				} else {
					error "Champ $champ non conforme";
				}
			} else {
				error "Champ $champ non defini!";
			}
		}
	}
	if ( $len == $ops->{$op}->{'longueur'} ) {
		error "$op/$type/len: OK";
	} else {
		error "$op/$type/len: $len invalide";
	}
}

sub ajoute_champ($$$) {
	my ( $op, $champ, $val ) = @_;
	if ( $champ =~ /^PAD(\d+)$/ ) {
		my $len = $1;
		$fichier .= ' 'x$len;
		return;
	}

	my $res;

	my $champ_court = $champ;
	$champ_court =~ s/\/.*$//;
	my $syntaxe = $ops->{$op}->{'champs'}->{$champ_court};
	if ( ! defined $syntaxe ) {
		error "Le champ $champ, raccourci en $champ_court n'a pas de syntaxe...";
	}
	$syntaxe =~ /^(\d+)(N|AN|BQ)$/;
	my ( $len, $type ) = ( $1, $2 );
	
	__debug "Ajout du champ $champ, type $type, longueur $len\n";
	$val = '' if ( ! defined $val );
	my $l = length($val);
	if ( $l > $len ) {
		$val =~ s/^(.{$len})(.*)$/$1/;
		__debug "Champ trop long ($l>$len), devient <$val> (retire <$2>)";
	}
	if ( $type eq 'N' || $type eq 'BQ' ) {
		if ( defined $val ) {
			$res = '0'x($len-$l) . $val;
		} else {
			$res = ' 'x$len;
		}
	} else {
		$res = $val . ' 'x($len-$l);
	}
	__debug "Resultat: <$res>";
	$fichier.=$res;
}

sub ajoute_crlf() {
	$fichier.="\r\n";
}
