# This file is part of the "FDN tools" project.
# You can use and redistribute it under the terms of the
# GNU General Public License.
# (c) French Data Network 1997-2005
# Authors and contributors:
# - Benjamin Bayart
package FDN::Etebac::Rib;

use strict;
use Exporter ;

our @ISA = ( 'Exporter' ) ;
our @EXPORT = qw (
		verif_clef_rib
	        verif_clef_iban
                );
our @EXPORT_OK = qw ();

use FDN::Common::Logger;
use FDN::Common::Modules;
use FDN::Common::Constantes;

# Pour la gestion du RIB

sub verif_clef_rib($$$$) {
	my ( $banque, $guichet, $compte, $clef ) = @_;
	return FALSE if ( $banque == 0 );
	return FALSE if ( $compte == 0 );
	return FALSE if ( $clef == 0 );
	$compte =~ tr/ABCDEFGHIJKLMNOPQRSTUVWXYZ/12345678912345678923456789/;
	conduse('Math::BigInt');
	my $id = Math::BigInt->new($banque.$guichet.$compte.$clef);
	$id->bmod(97);
	if ( $id->is_zero() ) {
		return TRUE;
	} else {
		return FALSE;
	}
}

sub verif_clef_iban($) {
    my ( $iban ) = @_;
    # On recupere le pays et la clef qui sont au debut
    if ( !( $iban =~ m/^(\w{4})(.{10,30})$/ ) ) {
        # trop bizarre, on en veut pas
        return FALSE;
    }
    my $paysclef = $1;
    my $bban = $2;
    my $newiban = lc($bban.$paysclef);	# au cas ou, et on faitr le reste en minuscules
    my $i;
    my $subst;
    my $asubst;
    # On substitue les lettres par une valeur numerique litterale
    # A -> 10, B -> 11, etc.
    for ($i = ord('a'); $i <= ord('z'); $i++) {
        $subst = sprintf("%d", (10 + $i - ord('a')));
    	$asubst = chr($i);
        $newiban =~ s/$asubst/$subst/g;
    }
    conduse('Math::BigInt');
    my $id = Math::BigInt->new($newiban);
    my $reste = $id->bmod(97);
    if ( $reste->is_one() ) {
        return TRUE;
    } else {
        return FALSE;
    }
}

1;

